#!/usr/bin/env ruby
# $Id: starter_type3font.rb,v 1.8 2013/02/18 17:03:43 rp Exp $
# Type 3 font starter:
# Create a simple Type 3 font from vector data
#
# Define a type 3 font with the glyphs "l" and "space" and output text with
# that font. In addition the glyph ".notdef" is defined which any undefined
# character will be mapped to.
#
# Required software: PDFlib/PDFlib+PDI/PPS 9
# Required data: none

require './PDFlib'

# This is where the data files are. Adjust as necessary.
searchpath = "../data"
outfile = "starter_type3font.pdf"

begin
    p = PDFlib.new

	p.set_option("SearchPath={{" + searchpath +"}}")

	# This means we must check return values of load_font() etc.
	p.set_option("errorpolicy=return")

	# all strings are expected as utf8
	p.set_option("stringformat=utf8")

	if (p.begin_document(outfile, "") == -1)
	    raise "Error: " + p.get_errmsg()
	end

	p.set_info("Creator", "PDFlib Cookbook")
	p.set_info("Title", "Starter Type 3 Font")

	# Create the font "SimpleFont" containing the glyph "l",
	# the glyph "space" for spaces and the glyph ".notdef" for any
	# undefined character

	p.begin_font("SimpleFont", 0.001, 0.0, 0.0, 0.001, 0.0, 0.0, "")
        # glyph for .notdef */
        p.begin_glyph_ext(0x0000, "width=266 boundingbox={0 0 0 0}")
        p.end_glyph()

        # glyph for U+0020 space */
        p.begin_glyph_ext(0x0020, "width=266 boundingbox={0 0 0 0}")
        p.end_glyph()

        # glyph for U+006C "l" */
        p.begin_glyph_ext(0x006C, "width=266 boundingbox={0 0 266 570}")
        p.setlinewidth(20)

	x = 197
	y = 10
	p.moveto(x, y)
	y += 530
	p.lineto(x, y)
	x -= 64
	p.lineto(x, y)
	y -= 530
	p.moveto(x, y)
	x += 128
	p.lineto(x, y)

	p.stroke()
	p.end_glyph()

	p.end_font()

	# Start page
	p.begin_page_ext(0, 0, "width=300 height=200")

	# Load the new "SimpleFont" font
	font = p.load_font("SimpleFont", "winansi", "")
	if font == -1
		raise "Error: " + p.get_errmsg + "\n"
	end

	# Output the characters "l" and "space" of the "SimpleFont" font.
	# The character "x" is undefined and will be mapped to ".notdef"

	p.fit_textline("lll lllxlll", 100, 100, "font=" + font.to_s + \
	    " fontsize=40")

	p.end_page_ext("")

	p.end_document("")

rescue  PDFlibException => pe
  print "PDFlib exception occurred:\n"
  print "[" + pe.get_errnum.to_s + "] " + pe.get_apiname + \
		": " + pe.get_errmsg + "\n"
rescue  Exception => e
  print e.backtrace.join("\n") + "\n" + e.to_s
ensure
  p.delete() if p
end

# vim: sw=4
